#!python3
#distutils: language = c++

__all__ = (
    "bernoulli_variate", "uniform_int_variate", "generate_canonical", "uniform_real_variate",
    "binomial_variate", "negative_binomial_variate", "geometric_variate", "poisson_variate",
    "normal_variate", "lognormal_variate", "exponential_variate", "gamma_variate", "weibull_variate",
    "extreme_value_variate", "chi_squared_variate", "cauchy_variate", "fisher_f_variate", "student_t_variate",
    "distribution_timer", "distribution", "timer", "quick_test",
)


cdef extern from "RNG.hpp":
    int         rng_bernoulli       "RNG::bernoulli_variate"(double)
    long long   rng_uniform_int     "RNG::uniform_int_variate"(long long, long long)
    long long   rng_binomial        "RNG::binomial_variate"(long long, double)
    long long   rng_neg_binomial    "RNG::negative_binomial_variate"(long long, double)
    long long   rng_geometric       "RNG::geometric_variate"(double)
    long long   rng_poisson         "RNG::poisson_variate"(double)
    double      rng_canonical       "RNG::generate_canonical"()
    double      rng_uniform_real    "RNG::uniform_real_variate"(double, double)
    double      rng_exponential     "RNG::exponential_variate"(double)
    double      rng_gamma           "RNG::gamma_variate"(double, double)
    double      rng_weibull         "RNG::weibull_variate"(double, double)
    double      rng_normal          "RNG::normal_variate"(double, double)
    double      rng_lognormal       "RNG::lognormal_variate"(double, double)
    double      rng_extreme_value   "RNG::extreme_value_variate"(double, double)
    double      rng_chi_squared     "RNG::chi_squared_variate"(double)
    double      rng_cauchy          "RNG::cauchy_variate"(double, double)
    double      rng_fisher_f        "RNG::fisher_f_variate"(double, double)
    double      rng_student_t       "RNG::student_t_variate"(double)


def bernoulli_variate(ratio_of_truth) -> bool:
    return rng_bernoulli(ratio_of_truth) == 1

def uniform_int_variate(left_limit, right_limit) -> int:
    return rng_uniform_int(left_limit, right_limit)

def binomial_variate(number_of_trials, probability) -> int:
    return rng_binomial(number_of_trials, probability)

def negative_binomial_variate(number_of_trials, probability) -> int:
    return rng_neg_binomial(number_of_trials, probability)

def geometric_variate(probability) -> int:
    return rng_geometric(probability)

def poisson_variate(mean) -> int:
    return rng_poisson(mean)

def generate_canonical():
    return rng_canonical()

def uniform_real_variate(left_limit, right_limit) -> float:
    return rng_uniform_real(left_limit, right_limit)
    
def exponential_variate(lambda_rate) -> float:
    return rng_exponential(lambda_rate)

def gamma_variate(shape, scale) -> float:
    return rng_gamma(shape, scale)
    
def weibull_variate(shape, scale) -> float:
    return rng_weibull(shape, scale)
    
def normal_variate(mean, std_dev) -> float:
    return rng_normal(mean, std_dev)

def lognormal_variate(log_mean, log_deviation) -> float:
    return rng_lognormal(log_mean, log_deviation)
    
def extreme_value_variate(location, scale) -> float:
    return rng_extreme_value(location, scale)

def chi_squared_variate(degrees_of_freedom) -> float:
    return rng_chi_squared(degrees_of_freedom)
    
def cauchy_variate(location, scale) -> float:
    return rng_cauchy(location, scale)
    
def fisher_f_variate(degrees_of_freedom_1, degrees_of_freedom_2) -> float:
    return rng_fisher_f(degrees_of_freedom_1, degrees_of_freedom_2)
    
def student_t_variate(degrees_of_freedom) -> float:
    return rng_student_t(degrees_of_freedom)


# DISTRIBUTION & PERFORMANCE TEST SUITE #
import time as _time
import math as _math
import statistics as _statistics
import random as _random


def timer(func: staticmethod, *args, cycles=32, silent=False, **kwargs):
    def inner_timer():
        results = []
        for _ in range(cycles):
            start = _time.time_ns()
            for _ in range(cycles):
                _ = func(*args, **kwargs)
            end = _time.time_ns()
            t_time = end - start
            results.append(t_time / cycles)
        m = min(results)
        n = _statistics.stdev(results)
        return m, max(1, n)

    results = [inner_timer() for _ in range(cycles)]
    m, n = min(results, key=lambda x: x[1])
    if not silent:
        print(f"Typical Timing: {_math.ceil(m)} ± {_math.ceil(n)} ns")


def distribution(func: staticmethod, *args, num_cycles=102400, post_processor: staticmethod = None, **kwargs):
    results = [func(*args, **kwargs) for _ in range(num_cycles)]
    if type(results[0]) is list:
        for i, _ in enumerate(results):
            results[i] = results[i][0]
    try:
        stat_samples = results[:min(1024, num_cycles)]
        if type(stat_samples[0]) == type(""):
            stat_samples = list(map(float, stat_samples))
        ave = _statistics.mean(stat_samples)
        median_lo = _statistics.median_low(stat_samples)
        median_hi = _statistics.median_high(stat_samples)
        median = median_lo if median_lo == median_hi else (median_lo, median_hi)
        std_dev = _statistics.stdev(stat_samples, ave)
        output = (
            f" Minimum: {min(stat_samples)}",
            f" Median: {median}",
            f" Maximum: {max(stat_samples)}",
            f" Mean: {ave}",
            f" Std Deviation: {std_dev}",
        )
        print(f"Statistics of {len(stat_samples)} samples:")
        print("\n".join(output))
    except:
        pass
    if post_processor is None:
        processed_results = results
        print(f"Distribution of {num_cycles} samples:")
        unique_results = list(set(results))
    else:
        processed_results = list(map(post_processor, results))
        unique_results = list(set(processed_results))
        print(f"Post-processor distribution of {num_cycles} samples using {post_processor.__name__} method:")
    try:
        unique_results.sort()
    except TypeError:
        pass
    result_obj = {
        key: f"{processed_results.count(key) / (num_cycles / 100)}%" for key in unique_results
    }
    for key, val in result_obj.items():
        print(f" {key}: {val}")


def distribution_timer(func: staticmethod, *args, num_cycles=102400, label="", post_processor=None, **kwargs):
    def quote_str(value):
        return f'"{value}"' if type(value) is str else str(value)

    arguments = ', '.join([quote_str(v) for v in args] + [f'{k}={quote_str(v)}' for k, v in kwargs.items()])
    if label:
        print(f"Output Analysis: {label}")
    elif hasattr(func, "__qualname__"):
        print(f"Output Analysis: {func.__qualname__}({arguments})")
    elif hasattr(func, "__name__"):
        print(f"Output Analysis: {func.__name__}({arguments})")
    else:
        print(f"Output Analysis: {func}({arguments})")
    timer(func, *args, **kwargs)
    distribution(func, *args, num_cycles=num_cycles, post_processor=post_processor, **kwargs)
    print("")


# Quick Test Script #
def quick_test(n=10240):
    def floor_mod_10(x):
        return _math.floor(x) % 10

    print("\nQuick Test: RNG Storm Engine")
    print('=' * 73)

    start = _time.time()

    print("\nBoolean Variate Distributions\n")
    distribution_timer(bernoulli_variate, 0.0, num_cycles=n)
    distribution_timer(bernoulli_variate, 1/3, num_cycles=n)
    distribution_timer(bernoulli_variate, 1/2, num_cycles=n)
    distribution_timer(bernoulli_variate, 2/3, num_cycles=n)
    distribution_timer(bernoulli_variate, 1.0, num_cycles=n)

    print("\nInteger Variate Distributions\n")
    print("Base Case")
    distribution_timer(_random.randint, 1, 6, num_cycles=n)
    distribution_timer(uniform_int_variate, 1, 6, num_cycles=n)
    distribution_timer(binomial_variate, 4, 0.5, num_cycles=n)
    distribution_timer(negative_binomial_variate, 5, 0.75, num_cycles=n)
    distribution_timer(geometric_variate, 0.75, num_cycles=n)
    distribution_timer(poisson_variate, 4.5, num_cycles=n)

    print("\nFloating Point Variate Distributions\n")
    print("Base Case")
    distribution_timer(_random.random, num_cycles=n, post_processor=round)
    distribution_timer(generate_canonical, num_cycles=n, post_processor=round)
    distribution_timer(uniform_real_variate, 0.0, 10.0, num_cycles=n, post_processor=_math.floor)
    print("Base Case")
    distribution_timer(_random.expovariate, 1.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(exponential_variate, 1.0, num_cycles=n, post_processor=_math.floor)
    print("Base Case")
    distribution_timer(_random.gammavariate, 1.0, 1.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(gamma_variate, 1.0, 1.0, num_cycles=n, post_processor=_math.floor)
    print("Base Case")
    distribution_timer(_random.weibullvariate, 1.0, 1.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(weibull_variate, 1.0, 1.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(extreme_value_variate, 0.0, 1.0, num_cycles=n, post_processor=round)
    print("Base Case")
    distribution_timer(_random.gauss, 5.0, 2.0, num_cycles=n, post_processor=round)
    distribution_timer(normal_variate, 5.0, 2.0, num_cycles=n, post_processor=round)
    print("Base Case")
    distribution_timer(_random.lognormvariate, 1.6, 0.25, num_cycles=n, post_processor=round)
    distribution_timer(lognormal_variate, 1.6, 0.25, num_cycles=n, post_processor=round)
    distribution_timer(chi_squared_variate, 1.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(cauchy_variate, 0.0, 1.0, num_cycles=n, post_processor=floor_mod_10)
    distribution_timer(fisher_f_variate, 8.0, 8.0, num_cycles=n, post_processor=_math.floor)
    distribution_timer(student_t_variate, 8.0, num_cycles=n, post_processor=round)

    end = _time.time()
    duration = round(end - start, 4)
    print()
    print('=' * 73)
    print(f"Total Test Time: {duration} seconds")
